/*
*
* Copyright (c) 2005 Sun Microsystems, Inc.
* All rights reserved.
* You may not modify, use, reproduce, or distribute this software except in compliance with the terms
* of the License at: http://developer.sun.com/berkeley_license.html
* Author : Sameer Tyagi, Sun Microsystems s.t@sun.com
* "Realizing Strategies for Document-Based Web Services With JAX-WS 2.0"
*  http://java.sun.com/developer/technicalArticles/xml/jaxrpcpatterns
*/


import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.ws.Dispatch;
import javax.xml.ws.Service;
import javax.xml.ws.soap.SOAPBinding;
import java.io.File;
import java.net.URI;

public class DispatchClient {



    public static void main(String[] args) {
        String url = System.getProperty("serviceurl");
        System.out.println("url " +url);
        try {

	    QName qname= new QName("urn:ConcretePOService","IPurchaseOrderPort");
//        Service service = Service.create(qname);
        Service service = Service.create(new QName("urn:ConcretePOService","POService"));
        service.addPort(qname,new URI(SOAPBinding.SOAP11HTTP_BINDING),url);
        Dispatch<Source> sourceDispatch = service.createDispatch(qname, Source.class, Service.Mode.PAYLOAD);


        DOMSource request = new DOMSource(createSOAPMessage("request.xml"));
        Source result = sourceDispatch.invoke(request);
	
		printSourceToConsole(result);


        } catch (Exception jex) {
            jex.printStackTrace();
        }
    }


   /**
     * Method to create a SOAPElement from a file
     *
     * @return SOAPElement
     * @throws Exception
     */
    public static Element createSOAPMessage(String filename) {
        Document doc = readFileCreateDocument(filename);
//              printNodeToConsole(doc);
		return doc.getDocumentElement();
    }


    /**
     * Read a file and create a DOM out of it
     *
     * @param filename the file where the XML is stored
     * @return
     */
    private static Document readFileCreateDocument(String filename) {
        Document xml=null;
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);
            factory.setNamespaceAware(true);
// Use the factory to create a DOM parser
            DocumentBuilder parser = factory.newDocumentBuilder();
            xml = parser.parse(new File(filename));

        } catch (Exception e) {
            System.out.println("Couldnt read file " + filename + " " + e);
        }
       return xml;
    }



       public static void printNodeToConsole(Node n) {
        try {
            System.out.println("\n=============================XML Received from Server =========================================");
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer();
            // also print to console
            transformer.transform(new DOMSource(n), new StreamResult(System.out));
            System.out.println("\n======================================================================");
        } catch (Exception e) {
            System.out.println(e);
        }
    }


	     public static void printSourceToConsole(Source s) {
        try {
            System.out.println("\n=============================XML Received from Server =========================================");
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer();
            // also print to console
            transformer.transform(s, new StreamResult(System.out));
            System.out.println("\n======================================================================");
        } catch (Exception e) {
            System.out.println(e);
        }
    }
  
}
