/*
 * Copyright (c) Sun Microsystems All Rights Reserved.
 */

import org.w3c.dom.Document;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.soap.*;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.File;

public class TestClient {

    public static void main(String args[]) throws Exception {

//      String serviceURL = "http://localhost:9090/ifxservice-war/ifxservice";
        String serviceURL = "http://localhost:9090/jaxrpc-IFXServiceWrapper/ifxservice";



        try {
            //Create Soap Connection
            SOAPConnectionFactory scf =
                    SOAPConnectionFactory.newInstance();
            SOAPConnection con = scf.createConnection();

            //Create Message
            MessageFactory mf =
                    MessageFactory.newInstance();
            SOAPMessage msg = mf.createMessage();

            // Get the SOAPPart from the message
            SOAPPart soapPartInput = msg.getSOAPPart();
            SOAPBody body = soapPartInput.getEnvelope().getBody();

            body.addDocument(readFileCreateDocument("F:\\JAX-WS-Examples\\Provider\\ifxmessage.xml"));




            // Save the changes to the message
            msg.saveChanges();

            // Invoke the service synchronously and
            // get the reply
            SOAPMessage reply = con.call(msg, serviceURL);

            // Get the SOAP body from the response
            SOAPBody outSoapBody =
                    reply.getSOAPPart().getEnvelope().getBody();


            // this is a DOM  
            printNodeToConsole(outSoapBody);
            // Check to see if there is any SOAP Fault
            if (outSoapBody.hasFault()) {
                // Take any actions for the exception scenario
                SOAPFault fault = outSoapBody.getFault();

                // Log the fault details
                System.out.println("SOAP Fault occurred");
                System.out.println("Fault Code is " +
                        fault.getFaultCode());
                System.out.println("Fault String is " +
                        fault.getFaultString());
            } else {
                // Print the output SOAP envelope
                System.out.println(reply.getSOAPPart().getEnvelope());
            }

            // Close the connection
            con.close();
        } catch (SOAPException e) {
            // Print the stack trace if there are any
            // SOAPExceptions
            e.printStackTrace();
        }
    }

    private static Document readFileCreateDocument(String filename) throws Exception {
        Document xml = null;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setValidating(false);
        factory.setNamespaceAware(true);
// Use the factory to create a DOM parser
        DocumentBuilder parser = factory.newDocumentBuilder();
        xml = parser.parse(new File(filename));
        return xml;


    }


    public static void printNodeToConsole(org.w3c.dom.Node n) {
        try {
            System.out.println("\n=============================XML Received from Server =========================================");
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer();
            // also print to console
            transformer.transform(new DOMSource(n), new StreamResult(System.out));
            System.out.println("\n======================================================================");
        } catch (Exception e) {
            System.out.println(e);
        }
    }
}

